@@ -25,13 +25,13 @@
 #include "qgis_gui.h"
 #include "qgis_sip.h"
 #include "qgsdistancearea.h"
-#include "qgsexpressioncontextgenerator.h"
 #include "qgsexpressioncontext.h"
 #include "qgsfieldproxymodel.h"
 
 
 class QgsMapLayer;
 class QgsVectorLayer;
+class QgsExpressionContextGenerator;
 
 
 /**
@@ -73,7 +73,7 @@ class GUI_EXPORT QgsFieldExpressionWidget : public QWidget
     //! Returns the title used for the expression dialog
     const QString expressionDialogTitle() { return mExpressionDialogTitle; }
 
-    //! setFilters allows fitering according to the type of field
+    //! setFilters allows filtering according to the type of field
     void setFilters( QgsFieldProxyModel::Filters filters );
 
     /**
@@ -84,7 +84,7 @@ class GUI_EXPORT QgsFieldExpressionWidget : public QWidget
     void setAllowEmptyFieldName( bool allowEmpty );
 
     /**
-     * Returns true if the combo box allows the empty field ("not set") choice.
+     * Returns TRUE if the combo box allows the empty field ("not set") choice.
      * \see setAllowEmptyFieldName()
      * \since QGIS 3.4.6
      */
@@ -106,12 +106,12 @@ class GUI_EXPORT QgsFieldExpressionWidget : public QWidget
     QString currentField( bool *isExpression = nullptr, bool *isValid = nullptr ) const;
 
     /**
-      * Returns true if the current expression is valid
+      * Returns TRUE if the current expression is valid
       */
     bool isValidExpression( QString *expressionError = nullptr ) const;
 
     /**
-     * If the content is not just a simple field this method will return true.
+     * If the content is not just a simple field this method will return TRUE.
      */
     bool isExpression() const;
 
@@ -171,7 +171,7 @@ class GUI_EXPORT QgsFieldExpressionWidget : public QWidget
     void setAllowEvalErrors( bool allowEvalErrors );
 
   signals:
-    //! the signal is emitted when the currently selected field changes
+    //! Emitted when the currently selected field changes.
     void fieldChanged( const QString &fieldName );
 
     //! fieldChanged signal with indication of the validity of the expression
