@@ -167,7 +167,7 @@ class GUI_EXPORT QgsSearchWidgetWrapper : public QgsWidgetWrapper
     virtual QString expression() const = 0;
 
     /**
-     * If this is true, then this search widget should take effect directly
+     * If this is TRUE, then this search widget should take effect directly
      * when its expression changes
      */
     virtual bool applyDirectly() = 0;
@@ -180,7 +180,7 @@ class GUI_EXPORT QgsSearchWidgetWrapper : public QgsWidgetWrapper
      * \since QGIS 2.16
      */
     // TODO QGIS 4.0 - make pure virtual
-    virtual QString createExpression( FilterFlags flags ) const { Q_UNUSED( flags ); return QStringLiteral( "TRUE" ); }
+    virtual QString createExpression( FilterFlags flags ) const { Q_UNUSED( flags ) return QStringLiteral( "TRUE" ); }
 
     /**
      * Gets a field name or expression to use as field comparison.
@@ -207,6 +207,12 @@ class GUI_EXPORT QgsSearchWidgetWrapper : public QgsWidgetWrapper
      */
     void setAggregate( const QString &aggregate );
 
+    /**
+     * Returns the field index
+     * \since QGIS 3.10
+     */
+    int fieldIndex() const;
+
   public slots:
 
     /**
@@ -217,9 +223,9 @@ class GUI_EXPORT QgsSearchWidgetWrapper : public QgsWidgetWrapper
 
     /**
      * Toggles whether the search widget is enabled or disabled.
-     * \param enabled set to true to enable widget
+     * \param enabled set to TRUE to enable widget
      */
-    void setEnabled( bool enabled ) override { Q_UNUSED( enabled ); }
+    void setEnabled( bool enabled ) override { Q_UNUSED( enabled ) }
 
   signals:
 
