@@ -204,7 +204,7 @@ class DBPlugin(QObject):
         # First try with the new core API, if that fails, proceed with legacy code
         try:
             md = QgsProviderRegistry.instance().providerMetadata(self.providerName())
-            for name in md.dbConnections().keys():
+            for name in md.dbConnections(False).keys():
                 conn_list.append(createDbPlugin(self.typeName(), name))
         except (AttributeError, QgsProviderConnectionException):
             settings = QgsSettings()
