@@ -21,21 +21,19 @@ __author__ = 'Alexander Bruy'
 __date__ = 'September 2014'
 __copyright__ = '(C) 2014, Alexander Bruy'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '8a899c8758b30adc3f8d6df682a5702dd3bfe6f0'
-
 import os
 
-from qgis.core import (QgsProcessingUtils,
+from qgis.core import (QgsApplication,
+                       QgsProcessingUtils,
                        QgsFeatureSink,
                        QgsProcessingParameterFeatureSource,
                        QgsProcessingParameterField,
                        QgsProcessingParameterFolderDestination,
-                       QgsProcessingOutputFolder,
+                       QgsProcessingException,
                        QgsProcessingOutputMultipleLayers,
                        QgsExpression,
-                       QgsFeatureRequest)
+                       QgsFeatureRequest,
+                       QgsVectorFileWriter)
 
 from processing.algs.qgis.QgisAlgorithm import QgisAlgorithm
 from processing.tools.system import mkdir
@@ -70,6 +68,12 @@ class VectorSplit(QgisAlgorithm):
                                                                   self.tr('Output directory')))
         self.addOutput(QgsProcessingOutputMultipleLayers(self.OUTPUT_LAYERS, self.tr('Output layers')))
 
+    def icon(self):
+        return QgsApplication.getThemeIcon("/algorithms/mAlgorithmSplitLayer.svg")
+
+    def svgIconPath(self):
+        return QgsApplication.iconPath("/algorithms/mAlgorithmSplitLayer.svg")
+
     def name(self):
         return 'splitvectorlayer'
 
@@ -78,9 +82,17 @@ class VectorSplit(QgisAlgorithm):
 
     def processAlgorithm(self, parameters, context, feedback):
         source = self.parameterAsSource(parameters, self.INPUT, context)
+        if source is None:
+            raise QgsProcessingException(self.invalidSourceError(parameters, self.INPUT))
+
         fieldName = self.parameterAsString(parameters, self.FIELD, context)
         directory = self.parameterAsString(parameters, self.OUTPUT, context)
 
+        output_format = context.preferredVectorFormat()
+        if not output_format in QgsVectorFileWriter.supportedFormatExtensions():
+            # fallback to gpkg if preferred format is not available
+            output_format = 'gpkg'
+
         mkdir(directory)
 
         fieldIndex = source.fields().lookupField(fieldName)
@@ -97,7 +109,7 @@ class VectorSplit(QgisAlgorithm):
         for current, i in enumerate(uniqueValues):
             if feedback.isCanceled():
                 break
-            fName = u'{0}_{1}.shp'.format(baseName, str(i).strip())
+            fName = '{0}_{1}.{2}'.format(baseName, str(i).strip(), output_format)
             feedback.pushInfo(self.tr('Creating layer: {}').format(fName))
 
             sink, dest = QgsProcessingUtils.createFeatureSink(fName, context, fields, geomType, crs)
